@@ -544,7 +544,8 @@ void RadioView::noticeConnectedI (ISoundStreamServer *s, bool pointer_valid)
 bool RadioView::startRecordingWithFormat(
     SoundStreamID      id,
     const SoundFormat &/*proposed_format*/,
-    SoundFormat       &/*real_format*/)
+    SoundFormat       &/*real_format*/,
+    const QString     &/*filenameTemplate*/)
 {
     if (!id.isValid() || id != queryCurrentSoundStreamSinkID() || m_StreamID2MenuID.contains(id))
         return false;
@@ -774,7 +775,7 @@ void RadioView::noticeWidgetPluginShown(WidgetPluginBase *p, bool shown)
 {
     if (!m_manager || !p)
         return;
-    if ((WidgetPluginBase*)m_manager->getConfigDialog() == p) {
+    if (static_cast<WidgetPluginBase*>(m_manager->getConfigDialog()) == p) {
         btnConfigure->blockSignals(true);
         btnConfigure->setChecked(shown);
         btnConfigure->blockSignals(false);
