@@ -20,6 +20,8 @@
 #endif
 
 #include <QtCore/QList>
+#include <QtCore/QXmlStreamReader>
+#include <QtCore/QTextCodec>
 
 #include <kconfiggroup.h>
 #include <kiconloader.h>
@@ -27,6 +29,10 @@
 #include <klocale.h>
 #include <kio/jobclasses.h>
 #include <kio/job.h>
+#include <ktemporaryfile.h>
+#include <kencodingprober.h>
+
+#include <unistd.h>
 
 #include "stationlist.h"
 #include "utils.h"
@@ -53,8 +59,15 @@ PLUGIN_LIBRARY_FUNCTIONS(InternetRadio, PROJECT_NAME, i18n("Pseudo radio device
 
 InternetRadio::InternetRadio(const QString &instanceID, const QString &name)
   : PluginBase(instanceID, name, i18n("Internet Radio Plugin")),
+    m_powerOn(false),
     m_decoderThread(NULL),
     m_currentStation(InternetRadioStation()),
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    m_currentStreamIdx(-1),
+    m_currentStreamRetriesMax(2),
+    m_currentStreamRetriesLeft(-1),
+    m_randStreamIdxOffset(-1),
+#endif
     m_stereoFlag(false),
     m_muted(false),
     m_defaultPlaybackVolume(0.5),
@@ -65,8 +78,14 @@ InternetRadio::InternetRadio(const QString &instanceID, const QString &name)
     m_RDS_visible(false),
     m_RDS_StationName(QString()),
     m_RDS_RadioText(QString()),
+    m_maxStreamProbeSize(4096),
+    m_maxStreamAnalyzeTime(0.5),
     m_waitForBufferMinFill(true)/*,
-    m_MimetypeJob(NULL)*/
+    m_MimetypeJob(NULL)*/,
+    m_playlistJob(NULL)
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    , m_streamJob(NULL)
+#endif
 {
     m_SoundStreamSinkID   = createNewSoundStream(false);
     m_SoundStreamSourceID = m_SoundStreamSinkID;
@@ -157,9 +176,8 @@ bool InternetRadio::noticePlaybackChannelsChanged(const QString & client_id, con
 
 bool InternetRadio::noticeSoundStreamClosed(SoundStreamID id)
 {
-    if (id == m_SoundStreamSourceID) {
-        return true;
-    } else if (id == m_SoundStreamSinkID) {
+    if (id == m_SoundStreamSourceID || id == m_SoundStreamSinkID) {
+        powerOff();
         return true;
     } else {
         return false;
@@ -237,7 +255,7 @@ bool InternetRadio::powerOn ()
 
         unmuteSource  (m_SoundStreamSourceID);
         sendUnmuteSink(m_SoundStreamSourceID);
-        notifyPowerChanged(true);
+        notifyPowerChanged(isPowerOn());
         notifyStationChanged(m_currentStation);
         notifyURLChanged(m_currentStation.url(), &m_currentStation);
 
@@ -259,6 +277,11 @@ bool InternetRadio::powerOff ()
     if (! isPowerOn())
         return true;
 
+    loadPlaylistStopJob();
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    stopStreamDownload();
+#endif
+
     queryPlaybackVolume(m_SoundStreamSinkID, m_defaultPlaybackVolume);
     if (m_PlaybackMixerMuteOnPowerOff) {
       sendMuteSink(m_SoundStreamSourceID);
@@ -266,12 +289,17 @@ bool InternetRadio::powerOff ()
     muteSource  (m_SoundStreamSourceID);
     radio_done();
 
-    sendStopPlayback(m_SoundStreamSinkID);
-    sendStopCapture (m_SoundStreamSinkID);
-    closeSoundStream(m_SoundStreamSinkID);
-    closeSoundStream(m_SoundStreamSourceID);
-    m_SoundStreamSourceID = createNewSoundStream(m_SoundStreamSourceID, false);
-    m_SoundStreamSinkID   = m_SoundStreamSourceID;
+    sendStopRecording(m_SoundStreamSinkID);
+    sendStopPlayback (m_SoundStreamSinkID);
+    sendStopCapture  (m_SoundStreamSinkID);
+
+    SoundStreamID oldSourceID = m_SoundStreamSourceID;
+    SoundStreamID oldSinkID   = m_SoundStreamSinkID;
+    m_SoundStreamSourceID     = createNewSoundStream(m_SoundStreamSourceID, false);
+    m_SoundStreamSinkID       = m_SoundStreamSourceID;
+    closeSoundStream (oldSourceID);
+    closeSoundStream (oldSinkID);
+
     notifySoundStreamCreated(m_SoundStreamSourceID);
     notifyCurrentSoundStreamSinkIDChanged  (m_SoundStreamSinkID);
     notifyCurrentSoundStreamSourceIDChanged(m_SoundStreamSourceID);
@@ -317,7 +345,7 @@ bool InternetRadio::activateStation(const RadioStation &rs)
 
 bool InternetRadio::isPowerOn() const
 {
-    return m_decoderThread;
+    return m_powerOn;
 }
 
 
@@ -450,13 +478,31 @@ bool    InternetRadio::isSourceMuted(SoundStreamID id, bool &m) const
 
 // IInternetRadioCfg methods
 
+static inline void assignChannelIfValid(QString &dest_channel, const QString &test_channel, const QStringList &valid_channels)
+{
+    if (valid_channels.contains(test_channel) || !valid_channels.size()) {
+        dest_channel = test_channel;
+    }
+}
+
 
 bool  InternetRadio::setPlaybackMixer(const QString &soundStreamClientID, const QString &ch, bool muteOnPowerOff, bool force)
 {
-    bool change = m_PlaybackMixerID != soundStreamClientID || m_PlaybackMixerChannel != ch || muteOnPowerOff != m_PlaybackMixerMuteOnPowerOff;
+    QString old_channel           = m_PlaybackMixerChannel;
     m_PlaybackMixerID             = soundStreamClientID;
-    m_PlaybackMixerChannel        = ch;
     m_PlaybackMixerMuteOnPowerOff = muteOnPowerOff;
+    ISoundStreamClient *mixer    = getSoundStreamClientWithID(m_PlaybackMixerID);
+    QStringList         channels = mixer ? mixer->getPlaybackChannels() : QStringList();
+
+    if (channels.size()) {
+        assignChannelIfValid(m_PlaybackMixerChannel, channels[0], channels);  // lowest priority
+    }
+    assignChannelIfValid(m_PlaybackMixerChannel, "PCM",       channels);
+    assignChannelIfValid(m_PlaybackMixerChannel, "Wave",      channels);
+    assignChannelIfValid(m_PlaybackMixerChannel, "Master",    channels);
+    assignChannelIfValid(m_PlaybackMixerChannel, ch,          channels);  // highest priority
+
+    bool change = (m_PlaybackMixerID != soundStreamClientID) || (old_channel != m_PlaybackMixerChannel) || (muteOnPowerOff != m_PlaybackMixerMuteOnPowerOff);
 
     if (change || force) {
         if (isPowerOn()) {
@@ -506,6 +552,11 @@ void   InternetRadio::saveState (KConfigGroup &config) const
     config.writeEntry("defaultPlaybackVolume",       m_defaultPlaybackVolume);
     config.writeEntry("URL",                         m_currentStation.url());
     config.writeEntry("PowerOn",                     isPowerOn());
+    config.writeEntry("maxStreamProbeSize",          m_maxStreamProbeSize);
+    config.writeEntry("maxStreamAnalyzeTime",        m_maxStreamAnalyzeTime);
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    config.writeEntry("maxStreamRetries",            m_currentStreamRetriesMax);
+#endif
 
     saveRadioDeviceID(config);
 }
@@ -521,6 +572,14 @@ void   InternetRadio::restoreState (const KConfigGroup &config)
     bool    muteOnPowerOff        = config.readEntry ("PlaybackMixerMuteOnPowerOff", false);
     m_defaultPlaybackVolume       = config.readEntry ("defaultPlaybackVolume",       0.5);
 
+
+    m_maxStreamProbeSize          = config.readEntry ("maxStreamProbeSize",          4096);
+    m_maxStreamAnalyzeTime        = config.readEntry ("maxStreamAnalyzeTime",        0.5);
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    m_currentStreamRetriesMax     = config.readEntry ("maxStreamRetries",            2);
+#endif
+
+
     setPlaybackMixer(PlaybackMixerID, PlaybackMixerChannel, muteOnPowerOff, /* force = */ true);
 
     setURL(config.readEntry("URL", KUrl()), NULL);
@@ -579,50 +638,70 @@ bool InternetRadio::noticeStationsChanged(const StationList &sl)
 void InternetRadio::radio_init()
 {
     m_stereoFlag = false;
-    if (m_decoderThread) {
-        return; // should not happen
-    }
     freeAllBuffers(); // just to be sure;-)
 
 //     m_MimetypeJob = KIO::mimetype(m_currentStation.url());
 //     QObject::connect(m_MimetypeJob, SIGNAL(mimetype(KIO::Job *, const QString &)), this, SLOT(slotMimetypeResult(KIO::Job *, const QString &)));
 //     m_MimetypeJob->start();
 
-    m_decoderThread = new DecoderThread(this, m_currentStation, MAX_BUFFERS);
-    m_decoderThread->start();
+//     m_decoderThread = new DecoderThread(this, m_currentStation, MAX_BUFFERS, m_maxStreamProbeSize, m_maxStreamAnalyzeTime);
+//     m_decoderThread->start();
 
     m_waitForBufferMinFill = true;
+    m_powerOn              = true;
+
+    loadPlaylistStartJob();
     logDebug(QString("InternetRadio::radio_init"));
 }
 
 
-// void InternetRadio::slotMimetypeResult(KIO::Job *job, const QString &type)
-// {
-//     if (job && !job->error()) {
-//         logDebug(QString("InternetRadio::radio_init: mimetype for %1: %2").arg(m_currentStation.url().pathOrUrl()).arg(type));
-//     }
-// }
-//
+void InternetRadio::startDecoderThread()
+{
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+    startStreamDownload();
+#endif
+    if (m_decoderThread) {
+        m_decoderThread->quit();
+    }
+    m_decoderThread = new DecoderThread(this, m_currentStation, m_currentPlaylist, MAX_BUFFERS, m_maxStreamProbeSize, m_maxStreamAnalyzeTime);
+    QObject::connect(m_decoderThread, SIGNAL(finished()),   this, SLOT(slotDecoderThreadFinished()));
+    QObject::connect(m_decoderThread, SIGNAL(terminated()), this, SLOT(slotDecoderThreadFinished()));
+    m_decoderThread->start();
+}
+
+
+void InternetRadio::slotDecoderThreadFinished()
+{
+    // to avoid orphaned thread objects, we will delete finished threads here.
+    // we cannot call deleteLater from within the thread loop, since
+    // deleteLate will not schedule an event in the thread but in the main loop,
+    // which might cause a deletion before the thread really finished.
+    checkDecoderMessages();
+
+    DecoderThread *t = static_cast<DecoderThread*>(sender());
+    if (t == m_decoderThread) {
+        m_decoderThread = NULL;
+    }
+    t->deleteLater();
+}
+
 
 void InternetRadio::radio_done()
 {
-// /*    if (m_MimetypeJob) {
-//         delete m_MimetypeJob;
-//     }
-//     m_MimetypeJob = NULL;*/
-
-    m_stereoFlag = false;
-    if (m_decoderThread) {
-        m_decoderThread->setDone();
-    }
     freeAllBuffers();
+    m_powerOn    = false;
+    m_stereoFlag = false;
+    if (m_decoderThread && m_decoderThread->decoder()) {
+        m_decoderThread->decoder()->setDone();
+    }
     if (m_decoderThread) {
-        if (!m_decoderThread->wait(2000)) { // wait at max 2000 ms, otherwise we'll kill you ;-)
+        m_decoderThread->quit();
+/*        if (m_decoderThread->isRunning() && !m_decoderThread->wait(500)) { // wait at max 500 ms, otherwise we'll kill you ;-)
             m_decoderThread->terminate();
-            m_decoderThread->wait();
+            m_decoderThread->wait(1000);
             logWarning(i18n("I'm sorry, but I'd to kill the ffmpeg decoder thread. It might be a good idea to restart KRadio"));
-        }
-        delete m_decoderThread;
+        }*/
+        // delete m_decoderThread; // the thread will delete itself, but only when it really finished.
         m_decoderThread = NULL;
     }
 }
@@ -722,24 +801,16 @@ bool InternetRadio::noticeReadyForPlaybackData(SoundStreamID id, size_t free_siz
     if (!id.isValid() || id != m_SoundStreamSourceID)
         return false;
 
-    if (m_decoderThread && m_decoderThread->warning()) {
-        logWarning(m_decoderThread->warningString());
-    }
-    if (m_decoderThread && m_decoderThread->debug()) {
-        logDebug(m_decoderThread->debugString());
-    }
-    if (m_decoderThread && m_decoderThread->error()) {
-        logError(m_decoderThread->errorString());
-        powerOff();
+    if (!checkDecoderMessages()) {
         return false;
     }
 
     int min_size = m_waitForBufferMinFill ? MIN_BUFFERS4PLAYBACK : 1;
 
-    if (!m_decoderThread)
+    if (!m_decoderThread || !m_decoderThread->decoder())
         return false;
 
-    int n_bufs = m_decoderThread->availableBuffers();
+    int n_bufs = m_decoderThread->decoder()->availableBuffers();
 
     if (n_bufs < min_size) {
         m_waitForBufferMinFill = true;
@@ -750,9 +821,9 @@ bool InternetRadio::noticeReadyForPlaybackData(SoundStreamID id, size_t free_siz
     m_waitForBufferMinFill = false;
 
     size_t consumed_size = SIZE_T_DONT_CARE;
-    while ((n_bufs = m_decoderThread->availableBuffers()) && (free_size > 0) && (consumed_size != 0)) {
+    while (m_decoderThread && (n_bufs = m_decoderThread->decoder()->availableBuffers()) && (free_size > 0) && (consumed_size != 0)) {
 
-        DataBuffer         &buf           = m_decoderThread->getFirstBuffer();
+        DataBuffer         &buf           = m_decoderThread->decoder()->getFirstBuffer();
         const char         *data          = buf.currentPointer();
         size_t              size          = min(buf.remainingSize(), free_size);
                             consumed_size = SIZE_T_DONT_CARE;
@@ -773,8 +844,8 @@ bool InternetRadio::noticeReadyForPlaybackData(SoundStreamID id, size_t free_siz
         }
         free_size -= consumed_size;
         buf.addProcessedSize(consumed_size);
-        if (buf.remainingSize() <= 0) {
-            m_decoderThread->popFirstBuffer();
+        if (m_decoderThread && buf.remainingSize() <= 0) {
+            m_decoderThread->decoder()->popFirstBuffer();
         }
     }
     return true;
@@ -782,6 +853,22 @@ bool InternetRadio::noticeReadyForPlaybackData(SoundStreamID id, size_t free_siz
 
 
 
+bool InternetRadio::checkDecoderMessages()
+{
+    if (m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->warning()) {
+        logWarning(m_decoderThread->decoder()->warningString());
+    }
+    if (m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->debug()) {
+        logDebug(m_decoderThread->decoder()->debugString());
+    }
+    if (m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->error()) {
+        logError(m_decoderThread->decoder()->errorString());
+        powerOff();
+        return false;
+    }
+    return true;
+}
+
 
 
 bool InternetRadio::event(QEvent *_e)
@@ -789,28 +876,13 @@ bool InternetRadio::event(QEvent *_e)
     if (SoundStreamDecodingEvent::isSoundStreamDecodingEvent(_e)) {
         SoundStreamDecodingEvent *e  = static_cast<SoundStreamDecodingEvent*>(_e);
 
-        if (m_decoderThread) {
-            if (m_decoderThread->warning()) {
-                logWarning(m_decoderThread->warningString());
-            }
-            if (m_decoderThread && m_decoderThread->debug()) {
-                logDebug(m_decoderThread->debugString());
-            }
-            if (m_decoderThread->error()) {
-                logError(m_decoderThread->errorString());
-                powerOff();
-            } else {
-                if (e->type() == SoundStreamDecodingTerminated) {
-/*                } else if (!m_muted && e->type() == SoundStreamDecodingStep) {
-                    SoundStreamDecodingStepEvent *step = static_cast<SoundStreamDecodingStepEvent*>(e);
-                    logDebug(QString("InternetRadio::event: STORE: buf_size = %1, buf_count = %2").arg(step->size()).arg(m_DataQueue.size())+1);
+        checkDecoderMessages();
 
-                    char *data              = step->takeData();
-                    size_t s                = step->size();
-                    const SoundMetaData &md = step->metaData();
-                    const SoundFormat   &sf = step->getSoundFormat();
-                    m_DataQueue.push_back(DataBuffer(data, s, md, sf));
-*/                }
+        if (m_decoderThread && m_decoderThread->decoder()) {
+            if (e->type() == SoundStreamDecodingTerminated) {
+                DecoderThread *t = m_decoderThread;
+                m_decoderThread = NULL;
+                t->deleteLater();
             }
         }
         return true;
@@ -822,8 +894,8 @@ bool InternetRadio::event(QEvent *_e)
 
 void InternetRadio::freeAllBuffers()
 {
-    if (m_decoderThread) {
-        m_decoderThread->flushBuffers();
+    if (m_decoderThread && m_decoderThread->decoder()) {
+        m_decoderThread->decoder()->flushBuffers();
     }
 }
 
@@ -847,8 +919,11 @@ bool InternetRadio::setURL(const KUrl &url, const InternetRadioStation *rs)
 
     if (oldurl != url) {
         if (isPowerOn()) {
-            radio_done();
-            radio_init();
+            // with pulse audio, latency steadily increases if not powered off/on
+            //radio_done();
+            //radio_init();
+            powerOff();
+            powerOn();
         }
         notifyURLChanged(url, &m_currentStation);
     }
@@ -889,14 +964,17 @@ bool InternetRadio::startCaptureWithFormat(SoundStreamID      id,
     if (id != m_SoundStreamSourceID)
         return false;
 
-    if (!isPowerOn()) {
+    if(!isPowerOn()) {
         powerOn();
     }
-    for (int i = 0; i < 20 && m_decoderThread && !m_decoderThread->initDone(); ++i) {
-        usleep (200000);
-    } // wait max 4 secs
-    if (m_decoderThread && m_decoderThread->initDone()) {
-        real_format = m_decoderThread->soundFormat();
+
+    for (int i = 0; i < 20 && isPowerOn() && !(m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->initDone()); ++i) {
+        QEventLoop loop;
+        QTimer::singleShot(200, &loop, SLOT(quit()));
+        loop.exec();
+    } // wait max 4 secs*/
+    if (m_decoderThread && m_decoderThread->decoder() && m_decoderThread->decoder()->initDone()) {
+        real_format = m_decoderThread->decoder()->soundFormat();
         return true;
     } else {
         return false;
@@ -922,4 +1000,317 @@ const InternetRadioStation *InternetRadio::findMatchingStation(const StationList
 
 
 
+void InternetRadio::loadPlaylistStopJob()
+{
+    if (m_playlistJob) {
+        QObject::disconnect(m_playlistJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotPlaylistData(KIO::Job *, const QByteArray &)));
+        QObject::disconnect(m_playlistJob, SIGNAL(result(KJob *)),                        this, SLOT(slotPlaylistLoadDone(KJob *)));
+        m_playlistJob->kill();
+        m_playlistJob = NULL;
+    }
+}
+
+void InternetRadio::loadPlaylistStartJob()
+{
+    loadPlaylistStopJob();
+    // if the a new station is selected while the previous hasn't been loaded yet
+    m_playlistData   .clear();
+    m_currentPlaylist.clear();
+    if (getPlaylistClass().length()) {
+        m_playlistJob = KIO::get(m_currentStation.url(), KIO::NoReload, KIO::HideProgressInfo);
+        if (m_playlistJob) {
+            QObject::connect(m_playlistJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotPlaylistData(KIO::Job *, const QByteArray &)));
+            QObject::connect(m_playlistJob, SIGNAL(result(KJob *)),                         this, SLOT(slotPlaylistLoadDone(KJob *)));
+            m_playlistJob->start();
+            if (m_playlistJob->error()) {
+                logError(i18n("Failed to load playlist %1: %2").arg(m_currentStation.url().pathOrUrl()).arg(m_playlistJob->errorString()));
+                powerOff();
+            }
+        } else {
+            logError(i18n("Failed to start playlist download of %1: KIO::get returned NULL pointer").arg(m_currentStation.url().pathOrUrl()));
+            powerOff();
+        }
+    } else {
+        interpretePlaylistData(QByteArray());
+        startDecoderThread();
+    }
+}
+
+
+void InternetRadio::slotPlaylistData(KIO::Job *job, const QByteArray &data)
+{
+    if (job == m_playlistJob) {
+        m_playlistData.append(data);
+    }
+}
+
+
+void InternetRadio::slotPlaylistLoadDone(KJob *job)
+{
+    if (job == m_playlistJob) {
+        if (m_playlistJob->error()) {
+            logError(i18n("Failed to load playlist %1: %2").arg(m_currentStation.url().pathOrUrl()).arg(m_playlistJob->errorString()));
+            powerOff();
+        } else {
+            interpretePlaylistData(m_playlistData);
+            startDecoderThread();
+        }
+        m_playlistJob = NULL;
+    }
+    job->deleteLater();
+}
+
+
+QString InternetRadio::getPlaylistClass()
+{
+    QString plscls = m_currentStation.playlistClass();
+    QString path   = m_currentStation.url().path();
+    if        (plscls == "lsc" || (plscls == "auto" && path.endsWith(".lsc"))) {
+            return "lsc";
+    } else if (plscls == "m3u" || (plscls == "auto" && path.endsWith(".m3u"))) {
+            return "m3u";
+    } else if (plscls == "asx" || (plscls == "auto" && path.endsWith(".asx"))) {
+            return "asx";
+    } else if (plscls == "pls" || (plscls == "auto" && path.endsWith(".pls"))) {
+            return "pls";
+    } else {
+            return "";
+    }
+}
+
+
+void InternetRadio::interpretePlaylistData(const QByteArray &a)
+{
+//     IErrorLogClient::staticLogDebug("InternetRadio::interpretePlaylist");
+    QString plscls = getPlaylistClass();
+    m_currentPlaylist.clear();
+    if        (plscls == "lsc") {
+            interpretePlaylistLSC(a);
+    } else if (plscls == "m3u") {
+            interpretePlaylistM3U(a);
+    } else if (plscls == "asx") {
+            interpretePlaylistASX(a);
+    } else if (plscls == "pls") {
+            interpretePlaylistPLS(a);
+    } else {
+        m_currentPlaylist.append(m_currentStation.url());
+    }
+    if (!m_currentPlaylist.size()) {
+        logError(i18n("%1 does not contain any usable radio stream", m_currentStation.url().pathOrUrl()));
+    }
+
+/*    logDebug("Playlist:");
+    foreach (KUrl url, m_currentPlaylist) {
+        logDebug(url.pathOrUrl());
+    }*/
+}
+
+
+void InternetRadio::interpretePlaylistLSC(const QByteArray &a)
+{
+    interpretePlaylistM3U(a);
+    if (!m_currentPlaylist.size()) {
+        interpretePlaylistASX(a);
+    }
+}
+
+void InternetRadio::interpretePlaylistM3U(const QByteArray &playlistData)
+{
+    QStringList lines = QString(playlistData).split("\n");
+    foreach (QString line, lines) {
+        QString t = line.trimmed();
+        if (t.length() > 5 && !t.startsWith("#")) {
+            m_currentPlaylist.append(t);
+        }
+    }
+}
+
+
+void InternetRadio::interpretePlaylistPLS(const QByteArray &playlistData)
+{
+    KTemporaryFile tmpFile;
+    tmpFile.setAutoRemove(true);
+    if (!tmpFile.open()) {
+        logError(i18n("failed to create temporary file to store playlist data"));
+        return;
+    }
+    if (tmpFile.write(playlistData) != playlistData.size()) {
+        logError(i18n("failed to write temporary file to store playlist data"));
+        return;
+    }
+    tmpFile.close();
+
+    KConfig      cfg(tmpFile.fileName());
+
+    // mapping group names to lower case in order to be case insensitive
+    QStringList            groups = cfg.groupList();
+    QMap<QString, QString> group_lc_map;
+    QString                grp;
+    foreach(grp, groups) {
+        group_lc_map.insert(grp.toLower(), grp);
+    }
+
+    KConfigGroup cfggrp = cfg.group(group_lc_map["playlist"]);
+
+    // mapping entry keys to lower case in order to be case insensitive
+    QStringList keys = cfggrp.keyList();
+    QMap<QString, QString> key_lc_map;
+    QString key;
+    foreach(key, keys) {
+        key_lc_map.insert(key.toLower(), key);
+    }
+
+    unsigned int entries = cfggrp.readEntry(key_lc_map["numberofentries"], 0);
+    if (entries) {
+        for (unsigned int i = 0; i < entries; ++i) {
+            QString url = cfggrp.readEntry(key_lc_map[QString("file%1").arg(i)], QString());
+            if (url.length()) {
+                m_currentPlaylist.append(url);
+            }
+        }
+    }
+}
+
+
+void InternetRadio::interpretePlaylistASX(const QByteArray &rawData)
+{
+
+    KEncodingProber prober;
+    prober.feed(rawData);
+
+    QXmlStreamReader reader(QTextCodec::codecForName(prober.encoding())->toUnicode(rawData));
+
+    bool inEntry = false;
+
+    while (!reader.atEnd() && (reader.error() == QXmlStreamReader::NoError)) {
+        reader.readNext();
+        if (reader.isStartElement()) {
+            QStringRef name = reader.name();
+            if (name.toString().toLower() == "entry") {
+                inEntry = true;
+            }
+            else if (name.toString().toLower() == "ref" && inEntry) {
+                QXmlStreamAttributes attrs = reader.attributes();
+                QXmlStreamAttribute  attr;
+                foreach(attr, attrs) {
+                    if(attr.name().toString().toLower() == "href") {
+                        m_currentPlaylist.append(attr.value().toString());
+                    }
+                }
+            }
+        }
+        else if (reader.isEndElement()) {
+            QStringRef name = reader.name();
+            if (name == "entry") {
+                inEntry = false;
+            }
+        }
+    }
+
+    if (reader.error() != QXmlStreamReader::NoError) {
+        logError(i18n("error while reading asx file: ", reader.error()));
+    }
+}
+
+
+
+
+
+
+#ifndef INET_RADIO_STREAM_HANDLING_BY_DECODER_THREAD
+
+void InternetRadio::startStreamDownload()
+{
+    m_randStreamIdxOffset      = rint((m_currentPlaylist.size() - 1) * (float)rand() / (float)RAND_MAX);
+    m_currentStreamIdx         = 0;
+    m_currentStreamRetriesLeft = m_currentStreamRetriesMax;
+
+    tryNextStream();
+}
+
+
+void InternetRadio::tryNextStream()
+{
+    do {
+        if (--m_currentStreamRetriesLeft < 0) {
+            ++m_currentStreamIdx;
+            m_currentStreamRetriesLeft = m_currentStreamRetriesMax;
+        }
+        if (isPowerOn() && m_currentStreamIdx < m_currentPlaylist.size()) {
+            m_currentStreamUrl = m_currentPlaylist[(m_currentStreamIdx + m_randStreamIdxOffset) % m_currentPlaylist.size()];
+
+            // start download job
+            stopStreamDownload();
+            logDebug(i18n("opening stream %1", m_currentStreamUrl.pathOrUrl()));
+
+            m_streamJob = KIO::get(m_currentStreamUrl, KIO::NoReload, KIO::HideProgressInfo);
+            if (m_streamJob) {
+                QObject::connect(m_streamJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotStreamData(KIO::Job *, const QByteArray &)));
+                QObject::connect(m_streamJob, SIGNAL(result(KJob *)),                         this, SLOT(slotStreamDone(KJob *)));
+                m_streamJob->start();
+                if (m_streamJob->error()) {
+                    logError(i18n("Failed to start stream download of %1: %2").arg(m_currentStreamUrl.pathOrUrl()).arg(m_streamJob->errorString()));
+                    stopStreamDownload();
+                }
+            } else {
+                logError(i18n("Failed to start stream download of %1: KIO::get returned NULL pointer").arg(m_currentStreamUrl.pathOrUrl()));
+                stopStreamDownload();
+            }
+        } else {
+            logError(i18n("Failed to start any stream of %1").arg(m_currentStation.longName()));
+            powerOff();
+        }
+    } while (isPowerOn() && !m_streamJob);
+}
+
+
+void InternetRadio::stopStreamDownload()
+{
+    if (m_streamJob) {
+        QObject::disconnect(m_streamJob, SIGNAL(data  (KIO::Job *, const QByteArray &)), this, SLOT(slotStreamData(KIO::Job *, const QByteArray &)));
+        QObject::disconnect(m_streamJob, SIGNAL(result(KJob *)),                         this, SLOT(slotStreamDone(KJob *)));
+        m_streamJob->kill();
+        m_streamJob = NULL;
+    }
+}
+
+
+void InternetRadio::slotStreamData(KIO::Job *job, const QByteArray &data)
+{
+    if (m_streamJob == job) {
+//         logDebug(QString("stream data: %1 bytes").arg(data.size()));
+        if (m_decoderThread && m_decoderThread->decoder()) {
+            bool isfull = false;
+            m_decoderThread->decoder()->writeInputBuffer(data, isfull, m_currentStreamUrl);
+            if (isfull) {
+                m_streamJob->suspend();
+            }
+        }
+    }
+}
+
+
+void InternetRadio::slotStreamContinue()
+{
+    if (m_streamJob) {
+        m_streamJob->resume();
+    }
+}
+
+
+void InternetRadio::slotStreamDone(KJob *job)
+{
+    if (m_streamJob == job) {
+        if (m_streamJob->error()) {
+            logError(i18n("Failed to load stream data for %1: %2").arg(m_currentStreamUrl.pathOrUrl()).arg(m_streamJob->errorString()));
+            tryNextStream();
+        }
+        stopStreamDownload();
+    }
+    job->deleteLater();
+}
+
+#endif
+
+
 #include "internetradio.moc"
