@@ -84,14 +84,16 @@ void LircSupport::LIRC_init_fd()
     fprintf (stderr, "%s\n", (const char*)i18n("initializing kradio lirc plugin").toUtf8());
 
     m_fd_lirc     = lirc_init(LIRCPROG, 1);
-    m_lirc_notify = new QSocketNotifier(m_fd_lirc, QSocketNotifier::Read, this);
-    if (m_lirc_notify)
-        QObject::connect(m_lirc_notify, SIGNAL(activated(int)), this, SLOT(slotLIRC(int)));
 
     if (m_fd_lirc == -1) {
+        m_lirc_notify = NULL;
         logWarning      (i18n("Initializing kradio lirc plugin failed"));
         staticLogWarning(i18n("Initializing kradio lirc plugin failed"));
     } else {
+        m_lirc_notify = new QSocketNotifier(m_fd_lirc, QSocketNotifier::Read, this);
+        if (m_lirc_notify)
+            QObject::connect(m_lirc_notify, SIGNAL(activated(int)), this, SLOT(slotLIRC(int)));
+
         logDebug      (i18n("Initializing kradio lirc plugin successful"));
         staticLogDebug(i18n("Initializing kradio lirc plugin successful"));
     }
@@ -217,7 +219,7 @@ void LircSupport::slotLIRC(int /*socket*/ )
     // we need this list to catch all commands as quickly as possible
     // in order to free all lirc stuff *before* we start running our own
     // code. Makes for some unknown reason a major difference e.g. for internet
-    // radio: it happend, that repeat codes haven't been decoded properly
+    // radio: it happened, that repeat codes haven't been decoded properly
     // plus getting the whole lirc stuff out of sync. basically only observed
     // together with lircrcd
     QList<tmp_code_struct> list;
